package main.java;

import javafx.scene.layout.BorderPane;
import javafx.scene.shape.Circle;
import javafx.scene.text.Text;

import main.java.shape.abs.DraggableShape;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

/**
 * This class represents a border pane with additional functionality to handle the user interaction more efficiently.
 */
public class CustomBorderPane extends BorderPane {

    public double startDragX;
    public double startDragY;
    public static final List<DraggableShape> DRAGGABLE_SHAPES = new ArrayList<>();



    private static CustomBorderPane INSTANCE;

    public static CustomBorderPane instance() {
        if(INSTANCE == null)
            INSTANCE = new CustomBorderPane();
        return INSTANCE;
    }

    public static void removeObjects(){
        INSTANCE.getChildren().removeAll(DRAGGABLE_SHAPES);
        DRAGGABLE_SHAPES.clear();
    }

    private CustomBorderPane(){

        super();
        this.setOnMouseMoved((e) -> {
            startDragX = e.getSceneX();
            startDragY = e.getSceneY();
        });

        this.setOnMouseDragged((e) -> {
            double x = e.getSceneX();
            double xDiff = x - startDragX;
            startDragX = x;

            double y = e.getSceneY();
            double yDiff = y - startDragY;
            startDragY = y;


            for(DraggableShape draggableShape : DRAGGABLE_SHAPES){
                draggableShape.dragX(xDiff);
                draggableShape.dragY(yDiff);
            }
        });
    }
}
