package main.java.datamodel;


import main.java.layout.Sorter;

import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedList;

/**
 * Inner datamodel to represent the data model
 */
public class Person {

    /**
     * used to determine a unique id
     */
    public static long ID_COUNTER = 0;
    /**
     * the id of the person
     */
    private long ID;
    /**
     * the name of the person
     */
    private String name;
    /**
     * the sex of the person either male or female
     */
    private Sex sex;
    /**
     * the birthdate of the person. can also be approximated
     */
    private Date date_of_birth;
    /**
     * false if the birthdate was not approximated
     * true if it was approximated
     */
    private boolean birthDateApproximated = false;
    /**
     * false if the death date was not approximated
     * true if it was approximated
     */
    private boolean deathDateApproximated = false;
    /**
     * the death of the person. can also be approximated
     */
    private Date date_of_death;
    private float currentDOI = 100;
    private LinkedList<Relationship> relationships = new LinkedList<>();








    public Person() {
        this.ID = ID_COUNTER++;
    }

    /**
     * Constructor for the person
     * @param name the name of the person
     * @param date_of_birth birthdate of the person
     * @param date_of_death death date of the person
     * @param sex the sex of the person
     */
    public Person(String name, Date date_of_birth, Date date_of_death, Sex sex) {
        this();
        this.name = name;
        this.date_of_birth = date_of_birth;
        this.date_of_death = date_of_death;
        this.sex = sex;
    }


    /**
     * used for the depth search
     */
    private boolean visited = false;

    /**
     * used for the depth search of the doi calculation
     */
    private boolean doiVisited = false;

    public boolean isVisited() {
        return visited;
    }

    public boolean isDoiVisited() {
        return doiVisited;
    }

    public void setDoiVisited(boolean doiVisited) {
        this.doiVisited = doiVisited;
    }

    public void setVisited(boolean visited) {
        this.visited = visited;
    }

    public LinkedList<Relationship> getRelationships() {
        return relationships;
    }
    public void addRelationship(Relationship relationship){
        this.relationships.add(relationship);
    }

    public static long getIDCounter() {
        return ID_COUNTER;
    }

    public static long getIdCounter() {
        return ID_COUNTER;

    }

    public static void setIdCounter(long idCounter) {
        Person.ID_COUNTER = idCounter;
    }



    public boolean isBirthDateApproximated() {
        return birthDateApproximated;
    }

    public void setBirthDateApproximated(boolean birthDateApproximated) {
        this.birthDateApproximated = birthDateApproximated;
    }



    public float getCurrentDOI() {
        return currentDOI;
    }

    public void setCurrentDOI(float currentDOI) {
        this.currentDOI = currentDOI;
        if(this.currentDOI <= 0){
            this.currentDOI = 0;
        }
    }



    public boolean isDeathDateApproximated() {
        return deathDateApproximated;
    }

    public void setDeathDateApproximated(boolean deathDateApproximated) {
        this.deathDateApproximated = deathDateApproximated;
    }



    public long getID() {
        return ID;
    }

    public void setID(long ID) {
        this.ID = ID;
    }



    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }



    public Date getDate_of_birth() {
        return date_of_birth;
    }

    public void setDate_of_birth(Date date_of_birth) {
        this.date_of_birth = date_of_birth;
    }



    public Date getDate_of_death() {
        return date_of_death;
    }

    public void setDate_of_death(Date date_of_death) {
        this.date_of_death = date_of_death;
    }



    public Sex getSex() {
        return sex;
    }

    public void setSex(Sex sex) {
        this.sex = sex;
    }



    @Override
    public String toString() {
        return "Person{" +
                "ID=" + ID +
                ", name='" + name + '\'' +
                ", sex=" + sex +
                ", date_of_birth=" + date_of_birth +
                ", birthDateApproximated=" + birthDateApproximated +
                ", deathDateApproximated=" + deathDateApproximated +
                ", date_of_death=" + date_of_death +
                ", currentDOI=" + currentDOI +
                '}';
    }


}
