package main.java.layout;


import main.java.Main;
import main.java.datamodel.*;
import main.java.datamodel.exceptions.WrongFilePathException;
import org.gedcom4j.model.Individual;

import java.util.*;

/**
 * A class performing the conversion of a gedcom file
 */
public class GedComConversion {
    /**
     * a static method which converts the gedcom to the localblock representation
     * the local blocks are persons, which are related by marriage (also transitive)
     */
    public static LinkedList<LocalBlock> getLocalBlocks(String path) throws WrongFilePathException {


        GedcomConverter gedcomConverter;

        if(path == null) {
            Main.path = "src/main/res/gedcom/royal92.ged";
            gedcomConverter = new GedcomConverter("src/main/res/gedcom/royal92.ged");
        }else{
            gedcomConverter = new GedcomConverter(path);
        }

        ConverterObject converterObject = gedcomConverter.parseGedcom();


        for(Relationship relationship : converterObject.getRelationship()){
            if(relationship.getPerson1().getDate_of_birth() != null && relationship.getPerson2().getDate_of_birth() != null) {
                Calendar calendar1 = Calendar.getInstance();
                calendar1.setTime(relationship.getPerson1().getDate_of_birth());
                Calendar calendar2 = Calendar.getInstance();
                calendar2.setTime(relationship.getPerson2().getDate_of_birth());
                //System.out.println(Math.abs(calendar1.get(Calendar.YEAR) - calendar2.get(Calendar.YEAR)));
                //System.out.println(relationship.getPerson1().getName() + "    " + relationship.getPerson2().getName());
            }
        }

        Approximation.approximate(converterObject);
        Sorter.sortOutAllFailedApproximations(converterObject);

        LinkedList<LocalBlock> blocks = new LinkedList<>();//stores all the local blocks
        for (Person person : converterObject.getPersons().values()) {
            LocalBlock block = new LocalBlock();
            dfsLocalBlock(person, block);
            if(!block.isEmpty())
                blocks.add(block);
        }

        Sorter.sortBlocks(blocks);
        Sorter.sortRelationships(converterObject.getRelationship());
        Sorter.sortRelationshipsInPersons(converterObject.getPersons());


        return blocks;
    }


    /**
     * this method traverses the persons structure in a depth first search manner and connects the persons, who are related by marriage to one local block
     * the visited value is set to true, so a person is only contained in one block
     * @param person the current person
     * @param block the clock which should contain the married (or transitively related) persons
     */
    private static void dfsLocalBlock(Person person, LocalBlock block){

        if(person.isVisited())
            return;

        block.add(person);
        person.setVisited(true);

        for(Relationship relationship : person.getRelationships()){
            if(relationship.getType() == RelationshipType.DIVORCE || relationship.getType() == RelationshipType.MARRIAGE){

                Person person1 = relationship.getPerson1();
                Person person2 = relationship.getPerson2();
                if(!person1.isVisited())
                    dfsLocalBlock(person1, block);
                if(!person2.isVisited())
                    dfsLocalBlock(person2, block);
            }
        }
    }
}