package main.java.shape;

import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Line;
import javafx.scene.shape.Rectangle;
import main.java.CustomBorderPane;
import main.java.datamodel.Sex;
import main.java.shape.abs.DraggableShape;
import main.java.utils.Colors;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents the line which marks the relationship between parent and child as dotted line.
 */
public class ChildLine extends Line implements DraggableShape {

    private static final float STROKE_WIDTH = 3;
    private static final float CIRCLE_RADIUS = 5;
    private static final float SQUARE_LENGTH = 10;

    private List<Circle> circles = new ArrayList<>();
    private Rectangle square;

    /**
     * The constructor, which registers the childLine to the borderPane.
     * @param x the x coordinate of the line
     * @param startY the initial height of the line
     * @param endY the final height of the line
     * @param childY the height of the child.
     * @param childSex the sex of the child.
     * @param circleHeights the heights where the circles of the parents will be placed.
     */
    public ChildLine(double x, double startY, double endY, double childY, Sex childSex, List<Double> circleHeights){

        super(x, startY, x, endY);
        Color stroke = childSex == Sex.MALE ? Colors.COLOR_M : Colors.COLOR_F;
        for(Double circleHeight : circleHeights){
            Circle circle = new Circle(x, circleHeight, CIRCLE_RADIUS);
            circle.setStroke(stroke);
            circle.setFill(Colors.CIRCLE_FILL);
            circles.add(circle);
        }
        square = new Rectangle(x - SQUARE_LENGTH/2.0, childY - SQUARE_LENGTH/2.0, SQUARE_LENGTH, SQUARE_LENGTH);
        square.setStroke(stroke);
        square.setFill(Colors.CIRCLE_FILL);

        this.setStrokeWidth(STROKE_WIDTH);
        this.setStroke(stroke);
        this.getStrokeDashArray().addAll(2d, 10d);

        CustomBorderPane.DRAGGABLE_SHAPES.add(this);
        CustomBorderPane.instance().getChildren().add(this);
        CustomBorderPane.instance().getChildren().add(square);
        for(Circle circle : circles)
            CustomBorderPane.instance().getChildren().add(circle);
    }


    @Override
    public void dragX(double xDiff){
        double startX = this.getStartX();
        double endX = this.getEndX();
        this.setStartX(startX + xDiff);
        this.setEndX(endX + xDiff);
        for(Circle circle : circles){
            double circleX = circle.getCenterX();
            circle.setCenterX(circleX + xDiff);
        }
        double squareX = square.getX();
        square.setX(squareX + xDiff);
    }

    @Override
    public void dragY(double yDiff){
        double startY = this.getStartY();
        double endY = this.getEndY();
        this.setStartY(startY + yDiff);
        this.setEndY(endY + yDiff);
        for(Circle circle : circles){
            double circleY = circle.getCenterY();
            circle.setCenterY(circleY + yDiff);
        }
        double squareY = square.getY();
        square.setY(squareY + yDiff);
    }
}
